#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# "W3C HTTPD Common Log File Format" module for log reporting system
# v1.0 - Initial version, Sep 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module

use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $from, $user, $authuser, $day, $monthmmm, $hour, $minute, $request, 
	$result, $size,	$method, $document, $kbsize, $month, $startdate, $enddate,
	$n, $logfilename, $dieopenmsg, $dieclosemsg, %monthnr, $config
	);
		
@dim = ( "from", "user", "authuser", "day", "month", "hour", "method", "document", "result", "kbsize");

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

ProcessClfLogFile ($logfilename);

GenerateReport ("APACHE Log file analysis",	{} );

###############
# SUBROUTINES #
###############

# ProcessClfLogFile : 
#	Processes logfile in CLF format and populates counter arrays
#	Parameter : filename
#
sub ProcessClfLogFile {

	my $logfile = shift;

	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;

		($from, $user, $authuser, $day, $monthmmm, $hour, $minute, $request, $result, $size) = 
			($line =~ m!(\S+) (\S+) (\S+) \[(\d\d)\/(\w\w\w)\/\d\d\d\d:(\d\d):(\d\d).+"(.+)" (\d+) (\S+)!) ;

		# parse request field
		($method, $document) = ($request ne "-") ? split (/ /, $request) : ("-", "-");
		
		# Some normalization
		($document) = split (/\?/, $document); # document w/o parameters
		$kbsize = int ($size / 1024); # Size in KB
		$month = $monthnr {$monthmmm}; # Month number
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		#DEBUG! print "$from, $user, $authuser, $day, $month, $hour, $minute, $method, $document, $result, $kbsize\n";
		
		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }

		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
