#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# "Microsoft IIS-W3C Extended Logfile Format (default) module for log reporting system
# v1.0 - Initial version, Sep 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nstart, $logfilename, $dieopenmsg, $dieclosemsg,
	$month, $day, $hour, $minute, $from, $user, $to, $port, $method, $document, $result,
	$enddate, $startdate
	);
	
@dim = ( "month", "day", "hour", "from", "user", "to", "port", "method", "document", "result");

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nstart = 0;

ProcessIisLogFile ($logfilename);

GenerateReport (
	"IIS Log file analysis",
	{
		"Number of (re)starts : " => $nstart / 4,
	}
);

###############
# SUBROUTINES #
###############

# ProcessIisLogFile : 
#	Processes IIS logfile and populates counter arrays
#	Parameter : log filename
#
sub ProcessIisLogFile {

	my $logfile = shift;
	
	my $nl = 0;
		
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;
		
		if (/^#/) {
			$nstart++;
			next LOOP;
		}
		
		$nl++;
		
		($month, $day, $hour, $minute, $from, $user, $to, $port, $method, $document, $result)= 
			($line =~ m"\d\d\d\d-(\d\d)-(\d\d) (\d\d):(\d\d)\S+ (\S+) (\S+) (\S+) (\d+) (\w+) (\S+) \S+ (\d+)") ;
				
		($startdate = "$month/$day $hour:$minute") if ($nl == 1);
				
		#DEBUG! print "$month, $day, $hour, $minute, $from, $user, $to, $port, $method, $document, $result\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
