#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# ipchains syslog module for log reporting system
# v1.1 - Handling mixed syslogs, report processed lines 
# v1.0 - Initial version, Nov 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nproc, $logfilename, $dieopenmsg, $dieclosemsg,
	$monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message,
	$month, %monthnr, $tmpn,
	$chain, $action, $interface, $protocol, $source, $sourceport, $destination, $destport, $size, $rule,
	$sizeKB, $enddate, $startdate
	);
	
@dim = ( 
	"month", "day", "hour", "server", "chain", "action", "interface", 
	"protocol", "source", "sourceport", "destination", "destport", "sizeKB", "rule"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessIpchainsFile ($logfilename);

GenerateReport (
	"IPCHAINS Log file analysis",
	{
		"Processed IPCHAINS lines:" => $nproc
	}
);

###############
# SUBROUTINES #
###############

# ProcessIpchainsFile : 
#	Processes Ipchains logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessIpchainsFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if !/Packet log:/; # Only ipchains lines !!
		
		$nproc++;

		my $line = $_;

		($monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*) (\S*): (.*)!) ;

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		$tmpn = index $sourcewpid, "[";
		$source = ($tmpn >= 0) ? substr ($sourcewpid, 0, $tmpn) : $sourcewpid;
		
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);
		
		($chain, $action, $interface, $protocol, $source, $sourceport, $destination, $destport, $size, $rule) =
			($message =~ m!\S+ \S+ (\S+) (\S+) (\S+) PROTO=(\d+) (\S+):(\d+) (\S+):(\d+) L=(\d+) \S+ \S+ \S+ \S+ \S* (\S*)!) ;

		$sizeKB = int ( $size / 1024);

		#DEBUG! print "$month, $day, $hour, $minute, $server, $source, $chain, $action, $interface, $protocol, $source, $sourceport, $destination, $destport, $sizeKB, $rule\n";
		
		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }

		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
