#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# iptables/netfilter syslog module for log reporting system
# v1.0 - Initial version, Dec 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nproc, $logfilename, $dieopenmsg, $dieclosemsg,
	$monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message,
	$month, %monthnr, $tmpn,
	$in, $out, $source, $destination, $size, $ttl, $protocol, $sourceport, $destport, $window,
	$sizeKB, $windowKB, $enddate, $startdate
	);

@dim = ( 
	"month", "day", "hour", "server", "in", "out", "source", "destination", 
	"sizeKB", "ttl", "protocol", "sourceport", "destport", "windowKB"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessIptablesFile ($logfilename);

GenerateReport (
	"IPTABLES Log file analysis",
	{
		"Processed IPTABLES lines: " => $nproc
	}
);

###############
# SUBROUTINES #
###############

# ProcessIptablesFile : 
#	Processes Iptables logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessIptablesFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if !/IN=/; # Only ipchains lines !!
		
		$nproc++;

		my $line = $_;

		($monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*) (\S*): (.*)!) ;

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		$tmpn = index $sourcewpid, "[";
		$source = ($tmpn >= 0) ? substr ($sourcewpid, 0, $tmpn) : $sourcewpid;
		
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);
		
		($in, $out, $source, $destination, $size, $ttl, $protocol, $sourceport, $destport, $window) =
			($message =~ m!.*IN=(\S*) OUT=(\S*) \S+ SRC=(\S+) DST=(\S+) LEN=(\d+) \S+ \S+ TTL=(\d+) .* PROTO=(\S*) SPT=(\d*) DPT=(\d*) WINDOW=(\d*)!) ;

		$sizeKB = int ( $size / 1024);
		$windowKB = int ( $window / 1024);

		#DEBUG! print "$month, $day, $hour, $minute, $server, $in, $out, $source, $destination, $size, $ttl, $protocol, $sourceport, $destport, $window\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
