#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# NetScreen syslog module for log reporting system
# v1.0 - Initial version, Jan 2003, Tevfik K. (tevfik@itefix.no)

use strict;
use integer;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

# Variable declarations
our (
	@dim, $n, $nproc, $nmsgrepeat, $ntraffic, $resultmsg, $nrepeats, $logfilename,
	$monthmmm, $day, $hour, $minute, $server, $syslogsource, $message,
	$duration, $policyid, $protocol, $direction, $action, $sent, $received, $source, 
	$destination, $description1, $sourceport, $destport, $month, %monthnr, $errortype, $description,
	$sentKB, $receivedKB, $startdate, $enddate, $severity, $dieclosemsg, $dieopenmsg,
	);

# Sets analysis parameters - specific for each module

@dim = ( 
	"month", "day", "hour", "server", "severity", "errortype", "duration", "policyid", 
	"protocol", "direction", "action", "sentKB", "receivedKB", "source", "destination",
	"sourceport", "destport", "description"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = $nmsgrepeat = $ntraffic = 0;

ProcessNetscreenFile ($logfilename);

GenerateReport (
	"NetScreen Log file analysis",
	{
		"Processed lines: " => $nproc,
		"-Last message repeated- lines: " => $nmsgrepeat,
		"Traffic lines: " => $ntraffic
	}
);

###############
# SUBROUTINES #
###############

# ProcessNetscreenFile : 
#	Processes NetScreen logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessNetscreenFile {

	open LOGFILE, shift or die $dieopenmsg;
	
	my $line = "";	
	my $i = my $nrepeat = 0;

LOOP: 
	while (<LOGFILE>) {
	
		$nproc++;
	
		$line = $_;

		# Check -last message repeated-
		if ($line =~ /last message repeated/) {
		
			($nrepeats) = ($line =~ m!.* repeated (\d+)!);
			
			GenerateTables ($nrepeats);	
			$nmsgrepeat++;
			next LOOP;
			
		} 

		($monthmmm, $day, $hour, $minute, $server, $syslogsource, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*) (\S*): (.*)!) ;

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);

		($severity, $errortype, $description) =
			($message =~ m!.*-(\S+)-(\S+): (.*)!) ;
		
		if ($errortype =~ /traffic/) {
		
			($duration, $policyid, $protocol, $direction, $action, $sent, $received, $source, $destination, $description1) =
				($description =~ m!.* duration=(\d+) policy_id=(\d+) .* proto=(\d+) direction=(\S+) action=(\S+) sent=(\d+) rcvd=(\d+) src=(\S+) dst=(\S+) (.*)!) ;

			$sentKB = int ( $sent / 1024);
			$receivedKB = int ( $received / 1024);
		
			if ($protocol == 6 || $protocol == 17) {
			
				($sourceport, $destport) = ($description1 =~ m!src_port=(\d+) dst_port=(\d+)!);
				
				$description = '-';
				
			} else {
				
				$description = $description1;
				$sourceport = $destport = '-';
				
			}
			
			$ntraffic++;
		} else {
		
			$duration = $policyid = $protocol = $direction = $action = '-';
			$sentKB = $receivedKB = $source = $destination = $sourceport = $destport = '-';
			
		}

		#DEBUG! print "$month, $day, $hour, $server, $severity, $errortype, $duration, $policyid\n$protocol, $direction, $action, $sentKB, $receivedKB, $source, $destination, $sourceport $destport, $description\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
 
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
