#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# NT Event Log module for log reporting system
# v1.0 - Initial version, Sep 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, %typetxt, $nerror, $nwarning, $logfilename, $dieopenmsg, $dieclosemsg,
	$month, $day, $hour, $minute, $ntype, $category, $event, $source, $user, $type,
	$enddate, $startdate
	);
	
@dim = ("day", "month", "hour", "type", "category", "event", "source", "user");

%typetxt = (
	"0" => "Success",
	"1" => "Error",
	"2" => "Warning",
	"4" => "Information",
	"8" => "Audit success",
	"10" => "Audit Failure"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nerror = $nwarning = 0;

ProcessNTEventLogFile ($logfilename);

GenerateReport (
	"NT Event Log analysis",
	{
		"Number of errors:" 	=> $nerror,
		"Number of warnings:" 	=> $nwarning
	}
);

###############
# SUBROUTINES #
###############

# ProcessNTEventLogFile : 
#	Processes NT eventlogs produced by 'dumpel' and populates counter arrays
#	Parameter : filename
#
sub ProcessNTEventLogFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;

		($month, $day, $hour, $minute, $ntype, $category, $event, $source, $user) =
		($line =~ m!(\d+)\/(\d+)\/.+\t(\d+):(\d+):.+\t(\d+)\t(\d+)\t(\d+)\t([\ \S]+)\t(\S+)!) ;

		($startdate = "$month/$day $hour:$minute") if ($. == 1);
		
		$nerror++ if ($ntype == 1);
		$nwarning++ if ($ntype == 2);
		
		# Type as string
		$type = $typetxt {$ntype};
		
		#DEBUG! print "$month, $day, $hour, $minute, $type, $category, $event, $source, $user\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
