#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Oracle listener module for log reporting system

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $logfilename, $dieopenmsg, $dieclosemsg, $resultmsg,
	$month, $day, $hour, $minute, $connectdata, $address, $sid, $event, $returncode, $monthmmm, %monthnr,
	$enddate, $startdate, $nproc
	);

@dim = ( "month", "day", "hour", "connectdata", "address", "sid", "event", "returncode" );

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessOracleListenerFile ($logfilename);

$resultmsg = sprintf "Processed lines: %d", $n;

GenerateReport (
	"Oracle Listener log file analysis",
	{
		$resultmsg => "",
	}
);

###############
# SUBROUTINES #
###############

# ProcessOracleListenerFile : 
#	Processes Oracle Listener file and populates counter arrays
#	Parameter : filename
#
sub ProcessOracleListenerFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {
	
		next if !/ADDRESS=/;	# Process only lines with address information
				
		my $line = $_;

		($day, $monthmmm, $hour, $minute, $connectdata, $address, $event, $sid, $returncode) =
			($line =~ m!(\d\d)-(\w\w\w)-\d\d\d\d (\d\d):(\d\d):\d\d \* (\S+) \* (\S+) \* (\S+) \* (\S+) \* (\S+)!) ;
		
		next if (!defined $day);
		
		$nproc++;
		
		# Some normalization
		$month = $monthnr {ucfirst (lc $monthmmm)}; # Month number
		$connectdata = substr ($connectdata, index ($connectdata, "(PROGRAM="));
		$connectdata =~ s/\(/\//g;
		$connectdata =~ s/\)//g;
		$address = substr ($address, index ($address, "(PROTOCOL="));
		$address = substr ($address, 0, index ($address, "(PORT="));
		$address =~ s/\(/\//g;
		$address =~ s/\)//g;

		#DEBUG! print "$day, $month, $hour, $connectdata, $address, $event, $sid, $returncode\n";
		
		($startdate = "$month/$day $hour:$minute") if ($nproc == 1);

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$enddate = 	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
