#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Sendmail syslog module for log reporting system

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nproc, $logfilename, $dieopenmsg, $dieclosemsg, $enddate, $startdate,
	%arsess, $month, $day, $hour, $minute, $from, $size, $recipients, $to, $delay, $status,
	$nproc, $n, $monthmmm, $session, $server, $sourcewpid, $message,
	$linetype, $nmessages
	);
	
@dim = ( 
	"month", "day", "hour", "from", "size", "recipients", "to", "delay", "status"	 
);


################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = 0;

ProcessSendmailFile ($logfilename);

GenerateReport (
	"Sendmail log file analysis",
	{
		"Number of lines processed: " => $nproc,
		"Number of messages processed: " => $nmessages,
	}
);

###############
# SUBROUTINES #
###############

# ProcessSendmailFile : 
#	Processes Sendmail logfile and populates counter arrays
#	Parameter : filename
#
sub ProcessSendmailFile {

	my $logfile = shift;

	my @larg = ();
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		next if !/sendmail/; # Only sendmail lines !!

		my $line = $_;

		($monthmmm, $day, $hour, $minute, $server, $sourcewpid, $session, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*) (\S*): (\S*): (.*)!) ;

		# Only traffic line are interesting
		$linetype = ($message =~ /^from=/) ? 'from' : (($message =~ /^to=/) ? 'to' : '');
		next if $linetype eq '';

		$nproc++;

		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		
		($startdate = "$month/$day $hour:$minute") 
			if ($nproc == 1);

		$arsess {$session}{'month'} = $month;
		$arsess {$session}{'day'} = $day;
		$arsess {$session}{'hour'} = $hour;

		#DEBUG! print "$month, $day, $hour\n";

		@larg = split /, /, $message;

		foreach my $elm (@larg) {
			
			my ($tkey, $tval) = split /=/, $elm, 2;
			my $tkey1 = '';

			SW1 : {
				if ($tkey eq 'from') 	{ $tkey1 = 'from'; last SW1; };
				if ($tkey eq 'size') 	{ $tkey1 = 'size'; last SW1; };
				if ($tkey eq 'nrcpts') 	{ $tkey1 = 'recipients'; last SW1; };
				if ($tkey eq 'to') 	{ $tkey1 = 'to'; last SW1; };
				if ($tkey eq 'delay') 	{ $tkey1 = 'delay'; last SW1; };
				if ($tkey eq 'stat') 	{ $tkey1 = 'status'; last SW1; };
			}
			
			next if exists $arsess {$session}{$tkey1};

			$arsess {$session}{$tkey1} = $tval;
		}
	
	}
	
	$enddate = "$month/$day $hour:$minute";
	$n = $.;
	close LOGFILE or die $dieclosemsg;

	# Generate tables
	foreach my $session (keys %arsess) {
	
		#DEBUG! print "$session :\n";
			
		$. = $nmessages += 1;	# $. to make verbose function works properly
		
		#populate variables
		my $hp = $arsess {$session};
		no strict 'refs';
		while (my ($tkey, $tval) = each %$hp) { $$tkey = $tval; };
		use strict;
		# some adjustments
		$size = int ($size / 1024); # KB values
		$from =~ s/[<>,]//g;	# remove unnecessary characters
		$to =~ s/[<>,]//g;

		STSW : {
			my $tstatus = lc $status;
			if ($tstatus =~ /sent/) { 
				$status = 'Sent'; last STSW;
			};
 			if ($tstatus =~ /deferred: connection refused/) { 
				$status = 'Deferred: Connection refused'; last STSW;
			};
			if ($tstatus =~ /deferred: connection timed out/) { 
				$status = 'Deferred: Connection timed out'; last STSW;
			};
			if ($tstatus =~ /deferred:.*no route to host/) { 
				$status = 'Deferred: No route to host'; last STSW;
			};
			if ($tstatus =~ /deferred:.*network is unreachable/) { 
				$status = 'Deferred: Network is unreachable'; last STSW;
			};
			if ($tstatus =~ /host unknown/) { 
				$status = 'Host unknown'; last STSW;
			};
		}

		#DEBUG!	print "\t$month, $day, $hour, $from, $to, $size, $delay, $status, $recipients\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
						
		GenerateTables ();
	}
}
