#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# SNORT module for log reporting system
# v1.0 - Initial version, 06.2002, Tevfik K. (www.itefix.no)
 
# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nalert, $nportdetect, $nportstatus, $nportend, $logfilename, $dieopenmsg, $dieclosemsg,
	$month, $day, $hour, $minute, $from, $totaltime, $totalhosts, $totaltcp, $totaludp, $event, $to, $l,
	$enddate, $startdate
	);

@dim = ( "month", "day", "hour", "from", "to", "event");

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nalert = $nportdetect = $nportstatus = $nportend = 0;

ProcessSnortLogFile ($logfilename);

GenerateReport (
	"SNORT Log file analysis",
	{
		"Number of alerts: " => $nalert,
		"Portscanning messages: " => "detect - $nportdetect, status - $nportstatus, end - $nportend"
	}
);

###############
# SUBROUTINES #
###############

# ProcessSnortLogFile : 
#	Processes snort logfile and populates counter arrays
#	Parameter : Snort log filename
#
sub ProcessSnortLogFile {

	my $logfile = shift;
		
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;
		
		($month, $day, $hour, $minute) = ($line =~ m"(\d\d)/(\d\d)-(\d\d):(\d\d)");
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		do { $nportstatus++ ; next LOOP } if /portscan status/;
		do { $nportdetect++ ; next LOOP } if /PORTSCAN DETECTED/;

		do {
			$nportend++;
			($month, $day, $hour, $from, $totaltime, $totalhosts, $totaltcp, $totaludp) =
				($line =~ m"(\d\d)/(\d\d)-(\d\d).*from (.*): TOTAL time\((.*)\) hosts\((.*)\) TCP\((.*)\) UDP\((.*)\)");
			next LOOP;
		} if /End of portscan/;

		$nalert++;
		
		# parse log line
		($month, $day, $hour, $event, $from, $to) = ($line =~ m"(\d\d)/(\d\d)-(\d\d).*\].*\]\s(.*)\s\[\*\*\].*\{.*\}\s(.*)\s->\s(.*)");
	
		# remove port part from from/to adressess if necessary
		if (($l = index ($from, ":")) > 0) { $from = substr ($from, 0, $l) };
		if (($l = index ($to, ":")) > 0) { $to = substr ($to, 0, $l) };
				
		#DEBUG! print "$month $day $hour $event $from $to\n";
		
		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }

		GenerateTables ();
						
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}

