#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Generic syslog module for log reporting system
# v1.0 - Initial version, Oct 2002, Tevfik K. (tevfik@itefix.no)

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nproc, $nmsgrepeat, $logfilename, $dieopenmsg, $dieclosemsg, $resultmsg,
	$monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message, $month, %monthnr, $tmpn, $source,
	$enddate, $startdate
	);

@dim = ( "month", "day", "hour", "server", "source", "message" );

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = $nmsgrepeat = 0;

ProcessSyslogFile ($logfilename);

$resultmsg = sprintf "Processed lines: %d, -Last message repeated- lines: %d", $nproc, $nmsgrepeat;

GenerateReport (
	"Generic syslog analysis",
	{
		$resultmsg => "",
	}
);

###############
# SUBROUTINES #
###############

# ProcessSyslogFile : 
#	Processes syslog file and populates counter arrays
#	Parameter : filename
#
sub ProcessSyslogFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {
	
		$nproc++;
	
		my $line = $_;

		# Check -last message repeated-
		if ($line =~ /last message repeated/) {
		
			my $nrepeats = ($line =~ m!.* repeated (\d+)!);
			
			GenerateTables ($nrepeats);	
			$nmsgrepeat++;
			next LOOP;
			
		} 

		($monthmmm, $day, $hour, $minute, $server, $sourcewpid, $message ) =		 
			($line =~ m!(\S+)\s+(\d+)\s+(\d\d):(\d\d):\d\d\s+(\S*) (\S*): (.*)!) ;
				
		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		
		$tmpn = index $sourcewpid, "[";
		
		$source = ($tmpn >= 0) ? substr ($sourcewpid, 0, $tmpn) : $sourcewpid;
		
		($startdate = "$month/$day $hour:$minute") if ($. == 1);
				
		#DEBUG! print "$month, $day, $hour, $minute, $server, $source, $message\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
