#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002-2003 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# wtmp module for log reporting system

# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $nreboot, $nshutdown, $logfilename, $dieopenmsg, $dieclosemsg, $resultmsg,
	$month, $day, $hour, $user, $tty, $from, $duration_h, $duration_m, $duration, $monthmmm, %monthnr,
	$enddate, $startdate, $nproc
	);

@dim = ( "month", "day", "hour", "user", "tty", "from", "duration" );

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = $nproc = $nreboot = $nshutdown = 0;

ProcessWtmpFile ($logfilename);

$resultmsg = sprintf "Processed lines: %d, Reboots: %d, Shutdowns: %d", $n,$nreboot, $nshutdown;

GenerateReport (
	"WTMP log file analysis",
	{
		$resultmsg => "",
	}
);

###############
# SUBROUTINES #
###############

# ProcessWtmpFile : 
#	Processes wtmp file and populates counter arrays
#	Parameter : filename
#
sub ProcessWtmpFile {

	my $logfile = shift;
	
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {
	
		next if /still logged in/;	# Skip active logins
		next if /wtmp begins/;
		
		my $line = $_;

		# Check -reboot- or -shutdown- records
		if ($line =~ /^reboot/) 	{	$nreboot++;		next LOOP; }
		if ($line =~ /^shutdown/) 	{	$nshutdown++;	next LOOP; }

		($user, $tty, $from, $monthmmm, $day, $hour, $duration_h, $duration_m ) =		 
			($line =~ m!(\S+)\s+(\S+)\s+(\S+)\s+\w\w\w (\S+)\s+(\d+)\s+(\d+).*\((\d\d):(\d\d)\)!) ;
		
		next if (!defined $user);
		
		$nproc++;
				
		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		$duration = int (($duration_h * 60 + $duration_m) / 10) * 10;
				
		#DEBUG! print "$user, $tty, $from, $month, $day, $hour, $duration\n";

		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }
		
		GenerateTables ();
	}
	
	$startdate = $enddate = "---";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}
