#############################################################################
#
# LOGREP - Logfile Extraction And Reporting System                          
# Copyright (C) 2002 Tevfik Karaglle                                   
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
#
#############################################################################

#########
#
# Xferlog module for log reporting system
 
# Sets analysis parameters - specific for each module
use strict;
use FindBin qw($Bin);
use lib "$Bin/util/perl";
use Logrep::Common;

our (
	@dim, $n, $logfilename, $dieopenmsg, $dieclosemsg,
	$monthmmm, $month, $day, $hour, $minute, $transfertimesecs, $transfertimemins, $remotehost, $filesize,
	$filesizekb, $filename, $transfertype, $specialaction, $direction, $accessmode, $username, 
	$authmethod, $authuser,
	$enddate, $startdate
);

@dim = (
	"month", "day", "hour", "transfertimemins", "remotehost", "filesizekb", "filename", 
	"transfertype", "specialaction", "direction", "accessmode", "username", "authmethod", 
	"authuser"
);

################
# MAIN PROGRAM #
################

SetCommonParameters ();

# initialize counters
$n = 0;

ProcessXferlogFile ($logfilename);

GenerateReport (
	"Xferlog analysis", { }
);

###############
# SUBROUTINES #
###############

# ProcessXferlogFile : 
#	Processes  XFerlog file and populates counter arrays
#	Parameter : filename
#
sub ProcessXferlogFile {

	my $logfile = shift;
		
	open LOGFILE, $logfile or die $dieopenmsg;

LOOP: 
	while (<LOGFILE>) {

		my $line = $_;

		($monthmmm, $day, $hour, $minute, $transfertimesecs, $remotehost, $filesize, $filename, $transfertype, $specialaction, $direction, $accessmode, $username, $authmethod, $authuser) = 
			($line =~ /\S+ (\S+)\s+(\d+) (\d\d):(\d\d)\S+ \S+ (\d+) (\S+) (\d+) (\S+) (\w) (\S+) (\w) (\w) (\S+) (\S+) (\S+)/);
		
		# Some normalization
		$month = $monthnr {$monthmmm}; # Month number
		$filesizekb = int ($filesize / 1024);	 # file size in kb
		$transfertimemins = int ($transfertimesecs / 60); # file size in minutes

		($startdate = "$month/$day $hour:$minute") if ($. == 1);

		#DEBUG! print "$month, $day, $hour, $minute, $transfertimemins, $remotehost, $filesizekb, $filename, $transfertype, $specialaction, $direction, $accessmode, $username, $authmethod, $authuser\n";
		
		foreach my $attrval (@dim) { $Logrep::Common::{$attrval} = $attrval; }

		GenerateTables ();
						
	}
	
	$enddate = "$month/$day $hour:$minute";

	$n = $.;
	
	close LOGFILE or die $dieclosemsg;
}

